<?php

namespace App\Controller\OgRepository\Image;

use App\Controller\OgRepository\AbstractOgRepositoryController;
use App\Entity\Image;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\HttpKernel\Attribute\AsController;
use Symfony\Component\Validator\Exception\ValidatorException;
use Symfony\Contracts\HttpClient\Exception\ClientExceptionInterface;
use Symfony\Contracts\HttpClient\Exception\RedirectionExceptionInterface;
use Symfony\Contracts\HttpClient\Exception\ServerExceptionInterface;
use Symfony\Contracts\HttpClient\Exception\TransportExceptionInterface;
use Symfony\Contracts\HttpClient\HttpClientInterface;

#[AsController]
class GetAction extends AbstractOgRepositoryController
{
    /**
     * @throws TransportExceptionInterface
     * @throws ServerExceptionInterface
     * @throws RedirectionExceptionInterface
     * @throws ClientExceptionInterface
     */
    public function __invoke(Image $data): JsonResponse
    {
        if (!$data->getImageFullsum()) {
            throw new ValidatorException('Fullsum is required');
        }

        $content = [];

        foreach ($data->getRepositories() as $repository) {
            try {
                $response = $this->createRequest('GET', 'http://' . $repository->getIp() . ':8006/ogrepository/v1/images/' . $data->getImageFullsum());
                $content[] = [
                    'repository' => $repository->getIp(),
                    'data' => $response
                ];
            } catch (\Exception $e) {
                $this->logger->error(sprintf('Error en repositorio %s: %s', $repository->getIp(), $e->getMessage()));
                $content[] = [
                    'repository' => $repository->getIp(),
                    'error' => $e->getMessage(),
                    'details' => $e->getMessage()
                ];
            }
        }

        return new JsonResponse(data: $content, status: Response::HTTP_OK);
    }
}