<?php

namespace App\Controller\OgRepository\Image;

use App\Controller\OgRepository\AbstractOgRepositoryController;
use App\Dto\Input\ExportImportImageRepositoryInput;
use App\Entity\Image;
use App\Entity\ImageRepository;
use App\Model\CommandTypes;
use App\Model\ImageStatus;
use App\Model\TraceStatus;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\HttpKernel\Attribute\AsController;
use Symfony\Component\Validator\Exception\ValidatorException;
use Symfony\Contracts\HttpClient\Exception\ClientExceptionInterface;
use Symfony\Contracts\HttpClient\Exception\RedirectionExceptionInterface;
use Symfony\Contracts\HttpClient\Exception\ServerExceptionInterface;
use Symfony\Contracts\HttpClient\Exception\TransportExceptionInterface;

#[AsController]
class ImportAction extends AbstractOgRepositoryController
{
    /**
     * @throws TransportExceptionInterface
     * @throws ServerExceptionInterface
     * @throws RedirectionExceptionInterface
     * @throws ClientExceptionInterface
     */
    public function __invoke(ExportImportImageRepositoryInput $input, ImageRepository $repository): JsonResponse
    {
        $images = $input->images;

        foreach ($images as $imageEntity) {
            /** @var Image $image */
            $image = $imageEntity->getEntity();

            if (!$image->getImageFullsum()) {
                throw new ValidatorException('Fullsum is required');
            }

            $params = [
                'json' => [
                    'image' => $image->getName().'.img',
                    'repo_ip' => $repository->getIp(),
                    'user' => 'opengnsys',
                ]
            ];

            $this->logger->info('Importing image', ['image' => $image->getName(), 'repository' => $repository->getIp()]);

            $repository = $image->getClient()->getRepository();

            $content = $this->createRequest('POST', 'http://'.$repository->getIp().':8006/ogrepository/v1/repo/images', $params);

            $inputData = [
                'imageName' => $image->getName(),
                'imageUuid' => $image->getUuid(),
                'repositoryUuid' => $repository->getUuid(),
            ];

            $this->createService->__invoke($image->getClient(), CommandTypes::IMPORT_IMAGE, TraceStatus::IN_PROGRESS, $content['job_id'], $inputData);

            $image->setStatus(ImageStatus::TRANSFERRING);
            $this->entityManager->persist($image);
            $this->entityManager->flush();
        }

        return new JsonResponse(data: [], status: Response::HTTP_OK);
    }
}